const React = require('react');
const styleSelector = require('./style/ConfigMenuBar');
const Setting = require('lib/models/Setting');

function ConfigMenuBarButton(props) {
	const style = styleSelector(null, props);

	const iconStyle = props.selected ? style.buttonIconSelected : style.buttonIcon;
	const labelStyle = props.selected ? style.buttonLabelSelected : style.buttonLabel;

	return React.createElement(
		'button',
		{ style: style.button, onClick: props.onClick },
		React.createElement('i', { style: iconStyle, className: props.iconName }),
		React.createElement(
			'span',
			{ style: labelStyle },
			props.label
		)
	);
}

function ConfigMenuBar(props) {
	const buttons = [];

	const style = styleSelector(null, props);

	for (const section of props.sections) {
		buttons.push(React.createElement(ConfigMenuBarButton, {
			selected: props.selection === section.name,
			theme: props.theme,
			key: section.name,
			iconName: Setting.sectionNameToIcon(section.name),
			label: Setting.sectionNameToLabel(section.name),
			onClick: () => {
				props.onSelectionChange({ section: section });
			}
		}));
	}

	return React.createElement(
		'div',
		{ style: style.root, className: 'config-menu-bar' },
		React.createElement(
			'div',
			{ style: style.barButtons },
			buttons
		)
	);
}

module.exports = ConfigMenuBar;
