"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const versionInfo_1 = require("lib/versionInfo");
const packageInfo = require('../packageInfo.js');
const ipcRenderer = require('electron').ipcRenderer;
class ErrorBoundary extends React.Component {
    constructor() {
        super(...arguments);
        this.state = { error: null, errorInfo: null };
    }
    componentDidCatch(error, errorInfo) {
        this.setState({ error: error, errorInfo: errorInfo });
    }
    componentDidMount() {
        const onAppClose = () => {
            ipcRenderer.send('asynchronous-message', 'appCloseReply', {
                canClose: true,
            });
        };
        ipcRenderer.on('appClose', onAppClose);
    }
    render() {
        if (this.state.error) {
            try {
                const output = [];
                output.push(React.createElement("section", { key: "message" },
                    React.createElement("h2", null, "Message"),
                    React.createElement("p", null, this.state.error.message)));
                output.push(React.createElement("section", { key: "versionInfo" },
                    React.createElement("h2", null, "Version info"),
                    React.createElement("pre", null, versionInfo_1.default(packageInfo).message)));
                if (this.state.error.stack) {
                    output.push(React.createElement("section", { key: "stacktrace" },
                        React.createElement("h2", null, "Stack trace"),
                        React.createElement("pre", null, this.state.error.stack)));
                }
                if (this.state.errorInfo) {
                    if (this.state.errorInfo.componentStack) {
                        output.push(React.createElement("section", { key: "componentStack" },
                            React.createElement("h2", null, "Component stack"),
                            React.createElement("pre", null, this.state.errorInfo.componentStack)));
                    }
                }
                return (React.createElement("div", { style: { overflow: 'auto', fontFamily: 'sans-serif', padding: '5px 20px' } },
                    React.createElement("h1", null, "Error"),
                    React.createElement("p", null, "Joplin encountered a fatal error and could not continue. To report the error, please copy the *entire content* of this page and post it on Joplin forum or GitHub."),
                    output));
            }
            catch (error) {
                return (React.createElement("div", null, JSON.stringify(this.state)));
            }
        }
        return this.props.children;
    }
}
exports.default = ErrorBoundary;
//# sourceMappingURL=ErrorBoundary.js.map