"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Note = require('lib/models/Note');
const BaseModel = require('lib/BaseModel');
const { _ } = require('lib/locale');
const eventManager = require('lib/eventManager');
const { time } = require('lib/time-utils');
exports.declaration = {
    name: 'editAlarm',
    label: () => _('Set alarm'),
    iconName: 'fa-clock',
};
exports.runtime = (comp) => {
    return {
        execute: ({ noteId }) => __awaiter(void 0, void 0, void 0, function* () {
            const note = yield Note.load(noteId);
            const defaultDate = new Date(Date.now() + 2 * 3600 * 1000);
            defaultDate.setMinutes(0);
            defaultDate.setSeconds(0);
            comp.setState({
                promptOptions: {
                    label: _('Set alarm:'),
                    inputType: 'datetime',
                    buttons: ['ok', 'cancel', 'clear'],
                    value: note.todo_due ? new Date(note.todo_due) : defaultDate,
                    onClose: (answer, buttonType) => __awaiter(void 0, void 0, void 0, function* () {
                        let newNote = null;
                        if (buttonType === 'clear') {
                            newNote = {
                                id: note.id,
                                todo_due: 0,
                            };
                        }
                        else if (answer !== null) {
                            newNote = {
                                id: note.id,
                                todo_due: answer.getTime(),
                            };
                        }
                        if (newNote) {
                            yield Note.save(newNote);
                            eventManager.emit('alarmChange', { noteId: note.id, note: newNote });
                        }
                        comp.setState({ promptOptions: null });
                    }),
                },
            });
        }),
        title: (props) => {
            if (!props.noteId || !props.notes)
                return null;
            const note = BaseModel.byId(props.notes, props.noteId);
            if (!note || !note.todo_due)
                return null;
            return time.formatMsToLocal(note.todo_due);
        },
        isEnabled: (props) => {
            const { notes, noteId } = props;
            if (!noteId || !notes)
                return false;
            const note = BaseModel.byId(notes, noteId);
            if (!note)
                return false;
            return !!note.is_todo && !note.todo_completed;
        },
        mapStateToProps: (state) => {
            return {
                noteId: state.selectedNoteIds.length === 1 ? state.selectedNoteIds[0] : null,
                notes: state.notes,
            };
        },
    };
};
//# sourceMappingURL=editAlarm.js.map