"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Note = require('lib/models/Note');
const { _ } = require('lib/locale');
const { shim } = require('lib/shim');
const { bridge } = require('electron').remote.require('./bridge');
const InteropServiceHelper = require('../../../InteropServiceHelper.js');
exports.declaration = {
    name: 'exportPdf',
    label: () => `PDF - ${_('PDF File')}`,
};
exports.runtime = (comp) => {
    return {
        execute: ({ noteIds }) => __awaiter(void 0, void 0, void 0, function* () {
            try {
                if (!noteIds.length)
                    throw new Error('No notes selected for pdf export');
                let path = null;
                if (noteIds.length === 1) {
                    path = bridge().showSaveDialog({
                        filters: [{ name: _('PDF File'), extensions: ['pdf'] }],
                        defaultPath: yield InteropServiceHelper.defaultFilename(noteIds[0], 'pdf'),
                    });
                }
                else {
                    path = bridge().showOpenDialog({
                        properties: ['openDirectory', 'createDirectory'],
                    });
                }
                if (!path)
                    return;
                for (let i = 0; i < noteIds.length; i++) {
                    const note = yield Note.load(noteIds[i]);
                    let pdfPath = '';
                    if (noteIds.length === 1) {
                        pdfPath = path;
                    }
                    else {
                        const n = yield InteropServiceHelper.defaultFilename(note.id, 'pdf');
                        pdfPath = yield shim.fsDriver().findUniqueFilename(`${path}/${n}`);
                    }
                    yield comp.printTo_('pdf', { path: pdfPath, noteId: note.id });
                }
            }
            catch (error) {
                console.error(error);
                bridge().showErrorMessageBox(error.message);
            }
        }),
        isEnabled: (props) => {
            return !!props.noteIds.length;
        },
        mapStateToProps: (state) => {
            return {
                noteIds: state.selectedNoteIds,
            };
        },
    };
};
//# sourceMappingURL=exportPdf.js.map