"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Folder = require('lib/models/Folder');
const Note = require('lib/models/Note');
const { _ } = require('lib/locale');
exports.declaration = {
    name: 'moveToFolder',
    label: () => _('Move to notebook'),
};
exports.runtime = (comp) => {
    return {
        execute: ({ noteIds }) => __awaiter(void 0, void 0, void 0, function* () {
            const folders = yield Folder.sortFolderTree();
            const startFolders = [];
            const maxDepth = 15;
            const addOptions = (folders, depth) => {
                for (let i = 0; i < folders.length; i++) {
                    const folder = folders[i];
                    startFolders.push({ key: folder.id, value: folder.id, label: folder.title, indentDepth: depth });
                    if (folder.children)
                        addOptions(folder.children, (depth + 1) < maxDepth ? depth + 1 : maxDepth);
                }
            };
            addOptions(folders, 0);
            comp.setState({
                promptOptions: {
                    label: _('Move to notebook:'),
                    inputType: 'dropdown',
                    value: '',
                    autocomplete: startFolders,
                    onClose: (answer) => __awaiter(void 0, void 0, void 0, function* () {
                        if (answer != null) {
                            for (let i = 0; i < noteIds.length; i++) {
                                yield Note.moveToFolder(noteIds[i], answer.value);
                            }
                        }
                        comp.setState({ promptOptions: null });
                    }),
                },
            });
        }),
        isEnabled: (props) => {
            return !!props.noteIds.length;
        },
        mapStateToProps: (state) => {
            return {
                noteIds: state.selectedNoteIds,
            };
        },
    };
};
//# sourceMappingURL=moveToFolder.js.map