"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const CommandService_1 = require("../../../lib/services/CommandService");
const Setting = require('lib/models/Setting');
const Note = require('lib/models/Note');
const Folder = require('lib/models/Folder');
const TemplateUtils = require('lib/TemplateUtils');
const { _ } = require('lib/locale');
exports.declaration = {
    name: 'newNote',
    label: () => _('New note'),
    iconName: 'fa-file',
};
exports.runtime = () => {
    return {
        execute: ({ template, isTodo }) => __awaiter(void 0, void 0, void 0, function* () {
            const folderId = Setting.value('activeFolderId');
            if (!folderId)
                return;
            const body = template ? TemplateUtils.render(template) : '';
            const defaultValues = Note.previewFieldsWithDefaultValues({ includeTimestamps: false });
            let newNote = Object.assign({}, defaultValues, {
                parent_id: folderId,
                is_todo: isTodo ? 1 : 0,
                body: body,
            });
            newNote = yield Note.save(newNote, { provisional: true });
            CommandService_1.utils.store.dispatch({
                type: 'NOTE_SELECT',
                id: newNote.id,
            });
        }),
        isEnabled: () => {
            const { folders, selectedFolderId } = CommandService_1.utils.store.getState();
            return !!folders.length && selectedFolderId !== Folder.conflictFolderId();
        },
        title: () => {
            return _('New note');
        },
    };
};
//# sourceMappingURL=newNote.js.map