"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const { _ } = require('lib/locale');
const Folder = require('lib/models/Folder');
const { bridge } = require('electron').remote.require('./bridge');
exports.declaration = {
    name: 'newNotebook',
    label: () => _('New notebook'),
    iconName: 'fa-book',
};
exports.runtime = (comp) => {
    return {
        execute: ({ parentId }) => __awaiter(void 0, void 0, void 0, function* () {
            comp.setState({
                promptOptions: {
                    label: _('Notebook title:'),
                    onClose: (answer) => __awaiter(void 0, void 0, void 0, function* () {
                        if (answer) {
                            let folder = null;
                            try {
                                const toSave = { title: answer };
                                if (parentId)
                                    toSave.parent_id = parentId;
                                folder = yield Folder.save(toSave, { userSideValidation: true });
                            }
                            catch (error) {
                                bridge().showErrorMessageBox(error.message);
                            }
                            if (folder) {
                                comp.props.dispatch({
                                    type: 'FOLDER_SELECT',
                                    id: folder.id,
                                });
                            }
                        }
                        comp.setState({ promptOptions: null });
                    }),
                },
            });
        }),
        mapStateToProps: (state) => {
            return {
                selectedNoteIds: state.selectedNoteIds,
                notes: state.notes,
            };
        },
        title: () => {
            return _('New notebook');
        },
    };
};
//# sourceMappingURL=newNotebook.js.map