"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Tag = require('lib/models/Tag');
const { _ } = require('lib/locale');
exports.declaration = {
    name: 'setTags',
    label: () => _('Tags'),
    iconName: 'fa-tags',
};
exports.runtime = (comp) => {
    return {
        execute: ({ noteIds }) => __awaiter(void 0, void 0, void 0, function* () {
            const tags = yield Tag.commonTagsByNoteIds(noteIds);
            const startTags = tags
                .map((a) => {
                return { value: a.id, label: a.title };
            })
                .sort((a, b) => {
                // sensitivity accent will treat accented characters as differemt
                // but treats caps as equal
                return a.label.localeCompare(b.label, undefined, { sensitivity: 'accent' });
            });
            const allTags = yield Tag.allWithNotes();
            const tagSuggestions = allTags.map((a) => {
                return { value: a.id, label: a.title };
            })
                .sort((a, b) => {
                // sensitivity accent will treat accented characters as differemt
                // but treats caps as equal
                return a.label.localeCompare(b.label, undefined, { sensitivity: 'accent' });
            });
            comp.setState({
                promptOptions: {
                    label: _('Add or remove tags:'),
                    inputType: 'tags',
                    value: startTags,
                    autocomplete: tagSuggestions,
                    onClose: (answer) => __awaiter(void 0, void 0, void 0, function* () {
                        if (answer !== null) {
                            const endTagTitles = answer.map(a => {
                                return a.label.trim();
                            });
                            if (noteIds.length === 1) {
                                yield Tag.setNoteTagsByTitles(noteIds[0], endTagTitles);
                            }
                            else {
                                const startTagTitles = startTags.map((a) => { return a.label.trim(); });
                                const addTags = endTagTitles.filter((value) => !startTagTitles.includes(value));
                                const delTags = startTagTitles.filter((value) => !endTagTitles.includes(value));
                                // apply the tag additions and deletions to each selected note
                                for (let i = 0; i < noteIds.length; i++) {
                                    const tags = yield Tag.tagsByNoteId(noteIds[i]);
                                    let tagTitles = tags.map((a) => { return a.title; });
                                    tagTitles = tagTitles.concat(addTags);
                                    tagTitles = tagTitles.filter((value) => !delTags.includes(value));
                                    yield Tag.setNoteTagsByTitles(noteIds[i], tagTitles);
                                }
                            }
                        }
                        comp.setState({ promptOptions: null });
                    }),
                },
            });
        }),
        isEnabled: (props) => {
            return !!props.noteIds.length;
        },
        mapStateToProps: (state) => {
            return { noteIds: state.selectedNoteIds };
        },
    };
};
//# sourceMappingURL=setTags.js.map