"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const react_1 = require("react");
const { _ } = require('lib/locale.js');
const { themeStyle } = require('lib/theme');
const DialogButtonRow = require('./DialogButtonRow.min');
const Countable = require('countable');
const markupLanguageUtils = require('lib/markupLanguageUtils');
let markupToHtml_ = null;
function markupToHtml() {
    if (markupToHtml_)
        return markupToHtml_;
    markupToHtml_ = markupLanguageUtils.newMarkupToHtml();
    return markupToHtml_;
}
function countElements(text, wordSetter, characterSetter, characterNoSpaceSetter, lineSetter) {
    Countable.count(text, (counter) => {
        wordSetter(counter.words);
        characterSetter(counter.all);
        characterNoSpaceSetter(counter.characters);
    });
    text === '' ? lineSetter(0) : lineSetter(text.split('\n').length);
}
function formatReadTime(readTimeMinutes) {
    if (readTimeMinutes < 1) {
        return '< 1';
    }
    return Math.ceil(readTimeMinutes).toString();
}
function NoteContentPropertiesDialog(props) {
    const theme = themeStyle(props.theme);
    const tableBodyComps = [];
    // For the source Markdown
    const [lines, setLines] = react_1.useState(0);
    const [words, setWords] = react_1.useState(0);
    const [characters, setCharacters] = react_1.useState(0);
    const [charactersNoSpace, setCharactersNoSpace] = react_1.useState(0);
    // For source with Markdown syntax stripped out
    const [strippedLines, setStrippedLines] = react_1.useState(0);
    const [strippedWords, setStrippedWords] = react_1.useState(0);
    const [strippedCharacters, setStrippedCharacters] = react_1.useState(0);
    const [strippedCharactersNoSpace, setStrippedCharactersNoSpace] = react_1.useState(0);
    const [strippedReadTime, setStrippedReadTime] = react_1.useState(0);
    // This amount based on the following paper:
    // https://www.researchgate.net/publication/332380784_How_many_words_do_we_read_per_minute_A_review_and_meta-analysis_of_reading_rate
    const wordsPerMinute = 250;
    react_1.useEffect(() => {
        countElements(props.text, setWords, setCharacters, setCharactersNoSpace, setLines);
    }, [props.text]);
    react_1.useEffect(() => {
        const strippedText = markupToHtml().stripMarkup(props.markupLanguage, props.text);
        countElements(strippedText, setStrippedWords, setStrippedCharacters, setStrippedCharactersNoSpace, setStrippedLines);
    }, [props.text]);
    react_1.useEffect(() => {
        const readTimeMinutes = strippedWords / wordsPerMinute;
        setStrippedReadTime(readTimeMinutes);
    }, [strippedWords]);
    const textProperties = {
        lines: lines,
        words: words,
        characters: characters,
        charactersNoSpace: charactersNoSpace,
    };
    const strippedTextProperties = {
        // The function stripMarkup() currently removes all new lines so we can't use the
        // strippedLines property. Instead we simply use the lines property which should
        // be a good approximation anyway.
        // Also dummy check to silence TypeScript warning
        lines: strippedLines === -5000 ? strippedLines : lines,
        words: strippedWords,
        characters: strippedCharacters,
        charactersNoSpace: strippedCharactersNoSpace,
    };
    const keyToLabel = {
        words: _('Words'),
        characters: _('Characters'),
        charactersNoSpace: _('Characters excluding spaces'),
        lines: _('Lines'),
    };
    const buttonRow_click = () => {
        props.onClose();
    };
    const labelCompStyle = Object.assign(Object.assign({}, theme.textStyle), { fontWeight: 'bold', width: '10em' });
    const controlCompStyle = Object.assign(Object.assign({}, theme.textStyle), { textAlign: 'center' });
    const createTableBodyRow = (key, value, strippedValue) => {
        const labelComp = React.createElement("td", { style: labelCompStyle }, keyToLabel[key]);
        const controlComp = React.createElement("td", { style: controlCompStyle }, value);
        const strippedControlComp = React.createElement("td", { style: controlCompStyle }, strippedValue);
        return (React.createElement("tr", { key: key },
            labelComp,
            controlComp,
            strippedControlComp));
    };
    const tableHeaderStyle = Object.assign(Object.assign({}, theme.textStyle), { textAlign: 'center' });
    const tableHeader = (React.createElement("tr", null,
        React.createElement("th", { style: tableHeaderStyle }),
        React.createElement("th", { style: tableHeaderStyle }, _('Editor')),
        React.createElement("th", { style: tableHeaderStyle }, _('Viewer'))));
    for (const key in textProperties) {
        const comp = createTableBodyRow(key, textProperties[key], strippedTextProperties[key]);
        tableBodyComps.push(comp);
    }
    const dialogBoxHeadingStyle = Object.assign(Object.assign({}, theme.dialogTitle), { textAlign: 'center' });
    return (React.createElement("div", { style: theme.dialogModalLayer },
        React.createElement("div", { style: theme.dialogBox },
            React.createElement("div", { style: dialogBoxHeadingStyle }, _('Statistics')),
            React.createElement("table", null,
                React.createElement("thead", null, tableHeader),
                React.createElement("tbody", null, tableBodyComps)),
            React.createElement("div", { style: labelCompStyle }, _('Read time: %s min', formatReadTime(strippedReadTime))),
            React.createElement(DialogButtonRow, { theme: props.theme, onClick: buttonRow_click, okButtonShow: false, cancelButtonLabel: _('Close') }))));
}
exports.default = NoteContentPropertiesDialog;
//# sourceMappingURL=NoteContentPropertiesDialog.js.map