"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
function cursorPositionToTextOffset(cursorPos, body) {
    if (!body)
        return 0;
    const noteLines = body.split('\n');
    let pos = 0;
    for (let i = 0; i < noteLines.length; i++) {
        if (i > 0)
            pos++; // Need to add the newline that's been removed in the split() call above
        if (i === cursorPos.line) {
            pos += cursorPos.ch;
            break;
        }
        else {
            pos += noteLines[i].length;
        }
    }
    return pos;
}
exports.cursorPositionToTextOffset = cursorPositionToTextOffset;
function usePrevious(value) {
    const ref = react_1.useRef();
    react_1.useEffect(() => {
        ref.current = value;
    });
    return ref.current;
}
exports.usePrevious = usePrevious;
function useScrollHandler(editorRef, webviewRef, onScroll) {
    const ignoreNextEditorScrollEvent_ = react_1.useRef(false);
    const scrollTimeoutId_ = react_1.useRef(null);
    const scheduleOnScroll = react_1.useCallback((event) => {
        if (scrollTimeoutId_.current) {
            clearTimeout(scrollTimeoutId_.current);
            scrollTimeoutId_.current = null;
        }
        scrollTimeoutId_.current = setTimeout(() => {
            scrollTimeoutId_.current = null;
            onScroll(event);
        }, 10);
    }, [onScroll]);
    const setEditorPercentScroll = react_1.useCallback((p) => {
        ignoreNextEditorScrollEvent_.current = true;
        if (editorRef.current) {
            editorRef.current.setScrollPercent(p);
            scheduleOnScroll({ percent: p });
        }
    }, [scheduleOnScroll]);
    const setViewerPercentScroll = react_1.useCallback((p) => {
        if (webviewRef.current) {
            webviewRef.current.wrappedInstance.send('setPercentScroll', p);
            scheduleOnScroll({ percent: p });
        }
    }, [scheduleOnScroll]);
    const editor_scroll = react_1.useCallback(() => {
        if (ignoreNextEditorScrollEvent_.current) {
            ignoreNextEditorScrollEvent_.current = false;
            return;
        }
        if (editorRef.current) {
            const percent = editorRef.current.getScrollPercent();
            setViewerPercentScroll(percent);
        }
    }, [setViewerPercentScroll]);
    const resetScroll = react_1.useCallback(() => {
        if (editorRef.current) {
            editorRef.current.setScrollPercent(0);
        }
    }, []);
    return { resetScroll, setEditorPercentScroll, setViewerPercentScroll, editor_scroll };
}
exports.useScrollHandler = useScrollHandler;
function useRootSize(dependencies) {
    const { rootRef } = dependencies;
    const [rootSize, setRootSize] = react_1.useState({ width: 0, height: 0 });
    react_1.useEffect(() => {
        if (!rootRef.current)
            return;
        const { width, height } = rootRef.current.getBoundingClientRect();
        if (rootSize.width !== width || rootSize.height !== height) {
            setRootSize({ width: width, height: height });
        }
    });
    return rootSize;
}
exports.useRootSize = useRootSize;
//# sourceMappingURL=index.js.map