"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const react_1 = require("react");
const types_1 = require("../../utils/types");
const resourceHandling_1 = require("../../utils/resourceHandling");
const useScroll_1 = require("./utils/useScroll");
const contextMenu_1 = require("../../utils/contextMenu");
const CommandService_1 = require("../../../../lib/services/CommandService");
const { MarkupToHtml } = require('lib/joplin-renderer');
const taboverride = require('taboverride');
const { reg } = require('lib/registry.js');
const { _, closestSupportedLocale } = require('lib/locale');
const BaseItem = require('lib/models/BaseItem');
const Resource = require('lib/models/Resource');
const { themeStyle, buildStyle } = require('lib/theme');
const { clipboard } = require('electron');
const supportedLocales = require('./supportedLocales');
function markupRenderOptions(override = null) {
    return Object.assign({ plugins: {
            checkbox: {
                renderingType: 2,
            },
            link_open: {
                linkRenderingType: 2,
            },
        }, replaceResourceInternalToExternalLinks: true }, override);
}
function findBlockSource(node) {
    const sources = node.getElementsByClassName('joplin-source');
    if (!sources.length)
        throw new Error('No source for node');
    const source = sources[0];
    return {
        openCharacters: source.getAttribute('data-joplin-source-open'),
        closeCharacters: source.getAttribute('data-joplin-source-close'),
        content: source.textContent,
        node: source,
        language: source.getAttribute('data-joplin-language') || '',
    };
}
function newBlockSource(language = '', content = '') {
    const fence = language === 'katex' ? '$$' : '```';
    const fenceLanguage = language === 'katex' ? '' : language;
    return {
        openCharacters: `\n${fence}${fenceLanguage}\n`,
        closeCharacters: `\n${fence}\n`,
        content: content,
        node: null,
        language: language,
    };
}
function findEditableContainer(node) {
    while (node) {
        if (node.classList && node.classList.contains('joplin-editable'))
            return node;
        node = node.parentNode;
    }
    return null;
}
function editableInnerHtml(html) {
    const temp = document.createElement('div');
    temp.innerHTML = html;
    const editable = temp.getElementsByClassName('joplin-editable');
    if (!editable.length)
        throw new Error(`Invalid joplin-editable: ${html}`);
    return editable[0].innerHTML;
}
function dialogTextArea_keyDown(event) {
    if (event.key === 'Tab') {
        window.requestAnimationFrame(() => event.target.focus());
    }
}
// Allows pressing tab in a textarea to input an actual tab (instead of changing focus)
// taboverride will take care of actually inserting the tab character, while the keydown
// event listener will override the default behaviour, which is to focus the next field.
function enableTextAreaTab(enable) {
    const textAreas = document.getElementsByClassName('tox-textarea');
    for (const textArea of textAreas) {
        taboverride.set(textArea, enable);
        if (enable) {
            textArea.addEventListener('keydown', dialogTextArea_keyDown);
        }
        else {
            textArea.removeEventListener('keydown', dialogTextArea_keyDown);
        }
    }
}
const joplinCommandToTinyMceCommands = {
    'textBold': { name: 'mceToggleFormat', value: 'bold' },
    'textItalic': { name: 'mceToggleFormat', value: 'italic' },
    'textLink': { name: 'mceLink' },
    'search': { name: 'SearchReplace' },
};
function styles_(props) {
    return buildStyle('TinyMCE', props.theme, ( /* theme:any */) => {
        return {
            disabledOverlay: {
                zIndex: 10,
                position: 'absolute',
                backgroundColor: 'white',
                opacity: 0.7,
                height: '100%',
                display: 'flex',
                flexDirection: 'column',
                alignItems: 'center',
                padding: 20,
                paddingTop: 50,
                textAlign: 'center',
                width: '100%',
            },
            rootStyle: Object.assign({ position: 'relative' }, props.style),
        };
    });
}
let loadedCssFiles_ = [];
let loadedJsFiles_ = [];
let dispatchDidUpdateIID_ = null;
let changeId_ = 1;
const TinyMCE = (props, ref) => {
    const [editor, setEditor] = react_1.useState(null);
    const [scriptLoaded, setScriptLoaded] = react_1.useState(false);
    const [editorReady, setEditorReady] = react_1.useState(false);
    const [draggingStarted, setDraggingStarted] = react_1.useState(false);
    const props_onMessage = react_1.useRef(null);
    props_onMessage.current = props.onMessage;
    const props_onDrop = react_1.useRef(null);
    props_onDrop.current = props.onDrop;
    const contextMenuActionOptions = react_1.useRef(null);
    const markupToHtml = react_1.useRef(null);
    markupToHtml.current = props.markupToHtml;
    const lastOnChangeEventInfo = react_1.useRef({
        content: null,
        resourceInfos: null,
        contentKey: null,
    });
    const rootIdRef = react_1.useRef(`tinymce-${Date.now()}${Math.round(Math.random() * 10000)}`);
    const editorRef = react_1.useRef(null);
    editorRef.current = editor;
    const styles = styles_(props);
    const theme = themeStyle(props.theme);
    const { scrollToPercent } = useScroll_1.default({ editor, onScroll: props.onScroll });
    const dispatchDidUpdate = (editor) => {
        if (dispatchDidUpdateIID_)
            clearTimeout(dispatchDidUpdateIID_);
        dispatchDidUpdateIID_ = setTimeout(() => {
            dispatchDidUpdateIID_ = null;
            if (editor && editor.getDoc())
                editor.getDoc().dispatchEvent(new Event('joplin-noteDidUpdate'));
        }, 10);
    };
    const insertResourcesIntoContent = react_1.useCallback((filePaths = null, options = null) => __awaiter(void 0, void 0, void 0, function* () {
        const resourceMd = yield resourceHandling_1.commandAttachFileToBody('', filePaths, options);
        if (!resourceMd)
            return;
        const result = yield props.markupToHtml(MarkupToHtml.MARKUP_LANGUAGE_MARKDOWN, resourceMd, markupRenderOptions({ bodyOnly: true }));
        editor.insertContent(result.html);
        // editor.fire('joplinChange');
        // dispatchDidUpdate(editor);
    }), [props.markupToHtml, editor]);
    const insertResourcesIntoContentRef = react_1.useRef(null);
    insertResourcesIntoContentRef.current = insertResourcesIntoContent;
    const onEditorContentClick = react_1.useCallback((event) => {
        const nodeName = event.target ? event.target.nodeName : '';
        if (nodeName === 'INPUT' && event.target.getAttribute('type') === 'checkbox') {
            editor.fire('joplinChange');
            dispatchDidUpdate(editor);
        }
        if (nodeName === 'A' && (event.ctrlKey || event.metaKey)) {
            const href = event.target.getAttribute('href');
            if (href.indexOf('#') === 0) {
                const anchorName = href.substr(1);
                const anchor = editor.getDoc().getElementById(anchorName);
                if (anchor) {
                    anchor.scrollIntoView();
                }
                else {
                    reg.logger().warn('TinyMce: could not find anchor with ID ', anchorName);
                }
            }
            else {
                props.onMessage({ channel: href });
            }
        }
    }, [editor, props.onMessage]);
    react_1.useImperativeHandle(ref, () => {
        return {
            content: () => __awaiter(void 0, void 0, void 0, function* () {
                if (!editorRef.current)
                    return '';
                return prop_htmlToMarkdownRef.current(props.contentMarkupLanguage, editorRef.current.getContent(), props.contentOriginalCss);
            }),
            resetScroll: () => {
                if (editor)
                    editor.getWin().scrollTo(0, 0);
            },
            scrollTo: (options) => {
                if (!editor)
                    return;
                if (options.type === types_1.ScrollOptionTypes.Hash) {
                    const anchor = editor.getDoc().getElementById(options.value);
                    if (!anchor) {
                        console.warn('Cannot find hash', options);
                        return;
                    }
                    anchor.scrollIntoView();
                }
                else if (options.type === types_1.ScrollOptionTypes.Percent) {
                    scrollToPercent(options.value);
                }
                else {
                    throw new Error(`Unsupported scroll options: ${options.type}`);
                }
            },
            supportsCommand: (name) => {
                // TODO: should also handle commands that are not in this map (insertText, focus, etc);
                return !!joplinCommandToTinyMceCommands[name];
            },
            execCommand: (cmd) => __awaiter(void 0, void 0, void 0, function* () {
                if (!editor)
                    return false;
                reg.logger().debug('TinyMce: execCommand', cmd);
                let commandProcessed = true;
                if (cmd.name === 'insertText') {
                    const result = yield markupToHtml.current(MarkupToHtml.MARKUP_LANGUAGE_MARKDOWN, cmd.value, { bodyOnly: true });
                    editor.insertContent(result.html);
                }
                else if (cmd.name === 'focus') {
                    editor.focus();
                }
                else if (cmd.name === 'dropItems') {
                    if (cmd.value.type === 'notes') {
                        const result = yield markupToHtml.current(MarkupToHtml.MARKUP_LANGUAGE_MARKDOWN, cmd.value.markdownTags.join('\n'), markupRenderOptions({ bodyOnly: true }));
                        editor.insertContent(result.html);
                    }
                    else if (cmd.value.type === 'files') {
                        insertResourcesIntoContentRef.current(cmd.value.paths, { createFileURL: !!cmd.value.createFileURL });
                    }
                    else {
                        reg.logger().warn('AceEditor: unsupported drop item: ', cmd);
                    }
                }
                else {
                    commandProcessed = false;
                }
                if (commandProcessed)
                    return true;
                if (!joplinCommandToTinyMceCommands[cmd.name]) {
                    reg.logger().warn('TinyMCE: unsupported Joplin command: ', cmd);
                    return false;
                }
                const tinyMceCmd = Object.assign({}, joplinCommandToTinyMceCommands[cmd.name]);
                if (!('ui' in tinyMceCmd))
                    tinyMceCmd.ui = false;
                if (!('value' in tinyMceCmd))
                    tinyMceCmd.value = null;
                editor.execCommand(tinyMceCmd.name, tinyMceCmd.ui, tinyMceCmd.value);
                return true;
            }),
        };
    }, [editor, props.contentMarkupLanguage, props.contentOriginalCss]);
    // -----------------------------------------------------------------------------------------
    // Load the TinyMCE library. The lib loads additional JS and CSS files on startup
    // (for themes), and so it needs to be loaded via <script> tag. Requiring it from the
    // module would not load these extra files.
    // -----------------------------------------------------------------------------------------
    const loadScript = (script) => __awaiter(void 0, void 0, void 0, function* () {
        return new Promise((resolve) => {
            let element = document.createElement('script');
            if (script.src.indexOf('.css') >= 0) {
                element = document.createElement('link');
                element.rel = 'stylesheet';
                element.href = script.src;
            }
            else {
                element.src = script.src;
                if (script.attrs) {
                    for (const attr in script.attrs) {
                        element[attr] = script.attrs[attr];
                    }
                }
            }
            element.id = script.id;
            element.onload = () => {
                resolve();
            };
            document.getElementsByTagName('head')[0].appendChild(element);
        });
    });
    react_1.useEffect(() => {
        let cancelled = false;
        function loadScripts() {
            return __awaiter(this, void 0, void 0, function* () {
                const scriptsToLoad = [
                    {
                        src: 'node_modules/tinymce/tinymce.min.js',
                        id: 'tinyMceScript',
                        loaded: false,
                    },
                    {
                        src: 'gui/NoteEditor/NoteBody/TinyMCE/plugins/lists.js',
                        id: 'tinyMceListsPluginScript',
                        loaded: false,
                    },
                ];
                for (const s of scriptsToLoad) {
                    if (document.getElementById(s.id)) {
                        s.loaded = true;
                        continue;
                    }
                    console.info('Loading script', s.src);
                    yield loadScript(s);
                    if (cancelled)
                        return;
                    s.loaded = true;
                }
                setScriptLoaded(true);
            });
        }
        loadScripts();
        return () => {
            cancelled = true;
        };
    }, []);
    react_1.useEffect(() => {
        if (!editorReady)
            return () => { };
        const element = document.createElement('style');
        element.setAttribute('id', 'tinyMceStyle');
        document.head.appendChild(element);
        element.appendChild(document.createTextNode(`
			.tox .tox-toolbar,
			.tox .tox-toolbar__overflow,
			.tox .tox-toolbar__primary,
			.tox-editor-header .tox-toolbar__primary,
			.tox .tox-toolbar-overlord,
			.tox.tox-tinymce-aux .tox-toolbar__overflow,
			.tox .tox-statusbar,
			.tox .tox-dialog__header,
			.tox .tox-dialog,
			.tox textarea,
			.tox input,
			.tox .tox-dialog__footer {
				background-color: ${theme.backgroundColor} !important;
			}

			.tox .tox-editor-header {
				border-top: 1px solid ${theme.dividerColor};
				border-bottom: 1px solid ${theme.dividerColor};
			}

			.tox .tox-tbtn,
			.tox .tox-tbtn svg,
			.tox .tox-dialog__header,
			.tox .tox-button--icon .tox-icon svg,
			.tox .tox-button.tox-button--icon .tox-icon svg,
			.tox textarea,
			.tox input,
			.tox .tox-label,
			.tox .tox-toolbar-label {
				color: ${theme.iconColor} !important;
				fill: ${theme.iconColor} !important;
			}

			.tox .tox-statusbar a,
			.tox .tox-statusbar__path-item,
			.tox .tox-statusbar__wordcount,
			.tox .tox-statusbar__path-divider {
				color: ${theme.color};
				fill: ${theme.color};
				opacity: 0.7;
			}

			.tox .tox-tbtn--enabled,
			.tox .tox-tbtn--enabled:hover {
				background-color: ${theme.selectedColor};
			}

			.tox .tox-button--naked:hover:not(:disabled) {
				background-color: ${theme.backgroundColor} !important;
			}

			.tox .tox-tbtn:hover {
				background-color: ${theme.backgroundHover};
				color: ${theme.colorHover};
				fill: ${theme.colorHover};
			}

			.tox .tox-toolbar__primary,
			.tox .tox-toolbar__overflow {
				background: none;
			}

			.tox-tinymce,
			.tox .tox-toolbar__group,
			.tox.tox-tinymce-aux .tox-toolbar__overflow,
			.tox .tox-dialog__footer {
				border-color: ${theme.dividerColor} !important;
			}

			.tox-tinymce {
				border-top: none !important;
			}
		`));
        return () => {
            document.head.removeChild(element);
        };
    }, [editorReady, props.theme]);
    // -----------------------------------------------------------------------------------------
    // Enable or disable the editor
    // -----------------------------------------------------------------------------------------
    react_1.useEffect(() => {
        if (!editor)
            return;
        editor.setMode(props.disabled ? 'readonly' : 'design');
    }, [editor, props.disabled]);
    // -----------------------------------------------------------------------------------------
    // Create and setup the editor
    // -----------------------------------------------------------------------------------------
    react_1.useEffect(() => {
        if (!scriptLoaded)
            return;
        loadedCssFiles_ = [];
        loadedJsFiles_ = [];
        function contextMenuItemNameWithNamespace(name) {
            // For unknown reasons, TinyMCE converts all context menu names to
            // lowercase when setting them in the init method, so we need to
            // make them lowercase too, to make sure that the update() method
            // addContextMenu is triggered.
            return (`joplin${name}`).toLowerCase();
        }
        const loadEditor = () => __awaiter(void 0, void 0, void 0, function* () {
            const contextMenuItems = contextMenu_1.menuItems();
            const contextMenuItemNames = [];
            for (const name in contextMenuItems)
                contextMenuItemNames.push(contextMenuItemNameWithNamespace(name));
            const language = closestSupportedLocale(props.locale, true, supportedLocales);
            const editors = yield window.tinymce.init({
                selector: `#${rootIdRef.current}`,
                width: '100%',
                body_class: 'jop-tinymce',
                height: '100%',
                resize: false,
                icons: 'Joplin',
                icons_url: 'gui/NoteEditor/NoteBody/TinyMCE/icons.js',
                plugins: 'noneditable link joplinLists hr searchreplace codesample table',
                noneditable_noneditable_class: 'joplin-editable',
                valid_elements: '*[*]',
                menubar: false,
                relative_urls: false,
                branding: false,
                target_list: false,
                table_resize_bars: false,
                language: ['en_US', 'en_GB'].includes(language) ? undefined : language,
                toolbar: 'bold italic | link joplinInlineCode joplinCodeBlock joplinAttach | numlist bullist joplinChecklist | h1 h2 h3 hr blockquote table joplinInsertDateTime',
                localization_function: _,
                contextmenu: contextMenuItemNames.join(' '),
                setup: (editor) => {
                    function openEditDialog(editable) {
                        const source = editable ? findBlockSource(editable) : newBlockSource();
                        editor.windowManager.open({
                            title: _('Edit'),
                            size: 'large',
                            initialData: {
                                codeTextArea: source.content,
                                languageInput: source.language,
                            },
                            onSubmit: (dialogApi) => __awaiter(this, void 0, void 0, function* () {
                                const newSource = newBlockSource(dialogApi.getData().languageInput, dialogApi.getData().codeTextArea);
                                const md = `${newSource.openCharacters}${newSource.content.trim()}${newSource.closeCharacters}`;
                                const result = yield markupToHtml.current(MarkupToHtml.MARKUP_LANGUAGE_MARKDOWN, md, { bodyOnly: true });
                                // markupToHtml will return the complete editable HTML, but we only
                                // want to update the inner HTML, so as not to break additional props that
                                // are added by TinyMCE on the main node.
                                if (editable) {
                                    editable.innerHTML = editableInnerHtml(result.html);
                                }
                                else {
                                    editor.insertContent(result.html);
                                }
                                dialogApi.close();
                                editor.fire('joplinChange');
                                dispatchDidUpdate(editor);
                            }),
                            onClose: () => {
                                enableTextAreaTab(false);
                            },
                            body: {
                                type: 'panel',
                                items: [
                                    {
                                        type: 'input',
                                        name: 'languageInput',
                                        label: 'Language',
                                        // Katex is a special case with special opening/closing tags
                                        // and we don't currently handle switching the language in this case.
                                        disabled: source.language === 'katex',
                                    },
                                    {
                                        type: 'textarea',
                                        name: 'codeTextArea',
                                        value: source.content,
                                    },
                                ],
                            },
                            buttons: [
                                {
                                    type: 'submit',
                                    text: 'OK',
                                },
                            ],
                        });
                        window.requestAnimationFrame(() => {
                            enableTextAreaTab(true);
                        });
                    }
                    editor.ui.registry.addButton('joplinAttach', {
                        tooltip: _('Attach file'),
                        icon: 'paperclip',
                        onAction: function () {
                            return __awaiter(this, void 0, void 0, function* () {
                                insertResourcesIntoContentRef.current();
                            });
                        },
                    });
                    editor.ui.registry.addButton('joplinCodeBlock', {
                        tooltip: _('Code Block'),
                        icon: 'code-sample',
                        onAction: function () {
                            return __awaiter(this, void 0, void 0, function* () {
                                openEditDialog(null);
                            });
                        },
                    });
                    editor.ui.registry.addToggleButton('joplinInlineCode', {
                        tooltip: _('Inline Code'),
                        icon: 'sourcecode',
                        onAction: function () {
                            editor.execCommand('mceToggleFormat', false, 'code', { class: 'inline-code' });
                        },
                        onSetup: function (api) {
                            api.setActive(editor.formatter.match('code'));
                            const unbind = editor.formatter.formatChanged('code', api.setActive).unbind;
                            return function () {
                                if (unbind)
                                    unbind();
                            };
                        },
                    });
                    editor.ui.registry.addButton('joplinInsertDateTime', {
                        tooltip: _('Insert Date Time'),
                        icon: 'insert-time',
                        onAction: function () {
                            CommandService_1.default.instance().execute('insertDateTime');
                        },
                    });
                    for (const itemName in contextMenuItems) {
                        const item = contextMenuItems[itemName];
                        const itemNameNS = contextMenuItemNameWithNamespace(itemName);
                        editor.ui.registry.addMenuItem(itemNameNS, {
                            text: item.label,
                            onAction: () => {
                                item.onAction(contextMenuActionOptions.current);
                            },
                        });
                        editor.ui.registry.addContextMenu(itemNameNS, {
                            update: function (element) {
                                let itemType = contextMenu_1.ContextMenuItemType.None;
                                let resourceId = '';
                                if (element.nodeName === 'IMG') {
                                    itemType = contextMenu_1.ContextMenuItemType.Image;
                                    resourceId = Resource.pathToId(element.src);
                                }
                                else if (element.nodeName === 'A') {
                                    resourceId = Resource.pathToId(element.href);
                                    itemType = resourceId ? contextMenu_1.ContextMenuItemType.Resource : contextMenu_1.ContextMenuItemType.Link;
                                }
                                else {
                                    itemType = contextMenu_1.ContextMenuItemType.Text;
                                }
                                contextMenuActionOptions.current = {
                                    itemType,
                                    resourceId,
                                    textToCopy: null,
                                    htmlToCopy: editor.selection ? editor.selection.getContent() : '',
                                    insertContent: (content) => {
                                        editor.insertContent(content);
                                    },
                                    isReadOnly: false,
                                };
                                return item.isActive(itemType, contextMenuActionOptions.current) ? itemNameNS : '';
                            },
                        });
                    }
                    // TODO: remove event on unmount?
                    editor.on('DblClick', (event) => {
                        const editable = findEditableContainer(event.target);
                        if (editable)
                            openEditDialog(editable);
                    });
                    // This is triggered when an external file is dropped on the editor
                    editor.on('drop', (event) => {
                        props_onDrop.current(event);
                    });
                    editor.on('ObjectResized', function (event) {
                        if (event.target.nodeName === 'IMG') {
                            editor.fire('joplinChange');
                            dispatchDidUpdate(editor);
                        }
                    });
                    editor.on('init', () => {
                        setEditorReady(true);
                    });
                    editor.on('SetContent', () => {
                        props_onMessage.current({ channel: 'noteRenderComplete' });
                    });
                },
            });
            setEditor(editors[0]);
        });
        loadEditor();
    }, [scriptLoaded]);
    // -----------------------------------------------------------------------------------------
    // Set the initial content and load the plugin CSS and JS files
    // -----------------------------------------------------------------------------------------
    const loadDocumentAssets = (editor, pluginAssets) => {
        // Note: The way files are cached is not correct because it assumes there's only one version
        // of each file. However, when the theme change, a new CSS file, specific to the theme, is
        // created. That file should not be loaded on top of the previous one, but as a replacement.
        // Otherwise it would do this:
        // - Try to load CSS for theme 1 => OK
        // - Try to load CSS for theme 2 => OK
        // - Try to load CSS for theme 1 => Skip because the file is in cache. As a result, theme 2
        //                                  incorrectly stay.
        // The fix would be to make allAssets() return a name and a version for each asset. Then the loading
        // code would check this and either append the CSS or replace.
        let docHead_ = null;
        function docHead() {
            if (docHead_)
                return docHead_;
            docHead_ = editor.getDoc().getElementsByTagName('head')[0];
            return docHead_;
        }
        const cssFiles = [
            'node_modules/@fortawesome/fontawesome-free/css/all.min.css',
            `gui/note-viewer/pluginAssets/highlight.js/${theme.codeThemeCss}`,
        ].concat(pluginAssets
            .filter((a) => a.mime === 'text/css')
            .map((a) => a.path)).filter((path) => !loadedCssFiles_.includes(path));
        const jsFiles = [].concat(pluginAssets
            .filter((a) => a.mime === 'application/javascript')
            .map((a) => a.path)).filter((path) => !loadedJsFiles_.includes(path));
        for (const cssFile of cssFiles)
            loadedCssFiles_.push(cssFile);
        for (const jsFile of jsFiles)
            loadedJsFiles_.push(jsFile);
        // console.info('loadDocumentAssets: files to load', cssFiles, jsFiles);
        if (cssFiles.length) {
            for (const cssFile of cssFiles) {
                const script = editor.dom.create('link', {
                    rel: 'stylesheet',
                    type: 'text/css',
                    href: cssFile,
                    class: 'jop-tinymce-css',
                });
                docHead().appendChild(script);
            }
        }
        if (jsFiles.length) {
            const editorElementId = editor.dom.uniqueId();
            for (const jsFile of jsFiles) {
                const script = editor.dom.create('script', {
                    id: editorElementId,
                    type: 'text/javascript',
                    src: jsFile,
                });
                docHead().appendChild(script);
            }
        }
    };
    react_1.useEffect(() => {
        if (!editor)
            return () => { };
        if (resourceHandling_1.resourcesStatus(props.resourceInfos) !== 'ready') {
            editor.setContent('');
            return () => { };
        }
        let cancelled = false;
        const loadContent = () => __awaiter(void 0, void 0, void 0, function* () {
            if (lastOnChangeEventInfo.current.content !== props.content || lastOnChangeEventInfo.current.resourceInfos !== props.resourceInfos) {
                const result = yield props.markupToHtml(props.contentMarkupLanguage, props.content, markupRenderOptions({ resourceInfos: props.resourceInfos }));
                if (cancelled)
                    return;
                editor.setContent(result.html);
                if (lastOnChangeEventInfo.current.contentKey !== props.contentKey) {
                    // Need to clear UndoManager to avoid this problem:
                    // - Load note 1
                    // - Make a change
                    // - Load note 2
                    // - Undo => content is that of note 1
                    //
                    // The doc is not very clear what's the different between
                    // clear() and reset() but it seems reset() works best, in
                    // particular for the onPaste bug.
                    //
                    // It seems the undo manager must be reset after having
                    // set the initial content (not before). Otherwise undoing multiple
                    // times would result in an empty note.
                    // https://github.com/laurent22/joplin/issues/3534
                    editor.undoManager.reset();
                }
                lastOnChangeEventInfo.current = {
                    content: props.content,
                    resourceInfos: props.resourceInfos,
                    contentKey: props.contentKey,
                };
            }
            yield loadDocumentAssets(editor, yield props.allAssets(props.contentMarkupLanguage));
            dispatchDidUpdate(editor);
        });
        loadContent();
        return () => {
            cancelled = true;
        };
    }, [editor, props.markupToHtml, props.allAssets, props.content, props.resourceInfos, props.contentKey]);
    react_1.useEffect(() => {
        if (!editor)
            return () => { };
        editor.getDoc().addEventListener('click', onEditorContentClick);
        return () => {
            editor.getDoc().removeEventListener('click', onEditorContentClick);
        };
    }, [editor, onEditorContentClick]);
    // This is to handle dropping notes on the editor. In this case, we add an
    // overlay over the editor, which makes it a valid drop target. This in
    // turn makes NoteEditor get the drop event and dispatch it.
    react_1.useEffect(() => {
        if (!editor)
            return () => { };
        function onDragStart() {
            setDraggingStarted(true);
        }
        function onDrop() {
            setDraggingStarted(false);
        }
        function onDragEnd() {
            setDraggingStarted(false);
        }
        document.addEventListener('dragstart', onDragStart);
        document.addEventListener('drop', onDrop);
        document.addEventListener('dragend', onDragEnd);
        return () => {
            document.removeEventListener('dragstart', onDragStart);
            document.removeEventListener('drop', onDrop);
            document.removeEventListener('dragend', onDragEnd);
        };
    }, [editor]);
    // -----------------------------------------------------------------------------------------
    // Handle onChange event
    // -----------------------------------------------------------------------------------------
    // Need to save the onChange handler to a ref to make sure
    // we call the current one from setTimeout.
    // https://github.com/facebook/react/issues/14010#issuecomment-433788147
    const props_onChangeRef = react_1.useRef();
    props_onChangeRef.current = props.onChange;
    const prop_htmlToMarkdownRef = react_1.useRef();
    prop_htmlToMarkdownRef.current = props.htmlToMarkdown;
    const nextOnChangeEventInfo = react_1.useRef(null);
    function execOnChangeEvent() {
        return __awaiter(this, void 0, void 0, function* () {
            const info = nextOnChangeEventInfo.current;
            if (!info)
                return;
            nextOnChangeEventInfo.current = null;
            const contentMd = yield prop_htmlToMarkdownRef.current(info.contentMarkupLanguage, info.editor.getContent(), info.contentOriginalCss);
            lastOnChangeEventInfo.current.content = contentMd;
            props_onChangeRef.current({
                changeId: info.changeId,
                content: contentMd,
            });
            dispatchDidUpdate(info.editor);
        });
    }
    // When the component unmount, we dispatch the change event
    // that was scheduled so that the parent component can save
    // the note.
    react_1.useEffect(() => {
        return () => {
            execOnChangeEvent();
        };
    }, []);
    const onChangeHandlerTimeoutRef = react_1.useRef(null);
    react_1.useEffect(() => {
        if (!editor)
            return () => { };
        function onChangeHandler() {
            // First this component notifies the parent that a change is going to happen.
            // Then the actual onChange event is fired after a timeout or when this
            // component gets unmounted.
            const changeId = changeId_++;
            props.onWillChange({ changeId: changeId });
            if (onChangeHandlerTimeoutRef.current)
                clearTimeout(onChangeHandlerTimeoutRef.current);
            nextOnChangeEventInfo.current = {
                changeId: changeId,
                editor: editor,
                contentMarkupLanguage: props.contentMarkupLanguage,
                contentOriginalCss: props.contentOriginalCss,
            };
            onChangeHandlerTimeoutRef.current = setTimeout(() => __awaiter(this, void 0, void 0, function* () {
                onChangeHandlerTimeoutRef.current = null;
                execOnChangeEvent();
            }), 1000);
        }
        function onExecCommand(event) {
            const c = event.command;
            if (!c)
                return;
            // We need to dispatch onChange for these commands:
            //
            // InsertHorizontalRule
            // InsertOrderedList
            // InsertUnorderedList
            // mceInsertContent
            // mceToggleFormat
            //
            // Any maybe others, so to catch them all we only check the prefix
            const changeCommands = ['mceBlockQuote', 'ToggleJoplinChecklistItem'];
            if (changeCommands.includes(c) || c.indexOf('Insert') === 0 || c.indexOf('mceToggle') === 0 || c.indexOf('mceInsert') === 0) {
                onChangeHandler();
            }
        }
        // Keypress means that a printable key (letter, digit, etc.) has been
        // pressed so we want to always trigger onChange in this case
        function onKeypress() {
            onChangeHandler();
        }
        // KeyUp is triggered for any keypress, including Control, Shift, etc.
        // so most of the time we don't want to trigger onChange. We trigger
        // it however for the keys that might change text, such as Delete or
        // Backspace. It's not completely accurate though because if user presses
        // Backspace at the beginning of a note or Delete at the end, we trigger
        // onChange even though nothing is changed. The alternative would be to
        // check the content before and after, but this is too slow, so let's
        // keep it this way for now.
        function onKeyUp(event) {
            if (['Backspace', 'Delete', 'Enter', 'Tab'].includes(event.key)) {
                onChangeHandler();
            }
        }
        function onPaste(event) {
            return __awaiter(this, void 0, void 0, function* () {
                const resourceMds = yield resourceHandling_1.handlePasteEvent(event);
                if (resourceMds.length) {
                    const result = yield markupToHtml.current(MarkupToHtml.MARKUP_LANGUAGE_MARKDOWN, resourceMds.join('\n'), markupRenderOptions({ bodyOnly: true }));
                    editor.insertContent(result.html);
                }
                else {
                    const pastedText = event.clipboardData.getData('text');
                    if (BaseItem.isMarkdownTag(pastedText)) { // Paste a link to a note
                        event.preventDefault();
                        const result = yield markupToHtml.current(MarkupToHtml.MARKUP_LANGUAGE_MARKDOWN, pastedText, markupRenderOptions({ bodyOnly: true }));
                        editor.insertContent(result.html);
                    }
                    else { // Paste regular text
                        // HACK: TinyMCE doesn't add an undo step when pasting, for unclear reasons
                        // so we manually add it here. We also can't do it immediately it seems, or
                        // else nothing is added to the stack, so do it on the next frame.
                        window.requestAnimationFrame(() => editor.undoManager.add());
                        onChangeHandler();
                    }
                }
            });
        }
        function onKeyDown(event) {
            // Handle "paste as text". Note that when pressing CtrlOrCmd+Shift+V it's going
            // to trigger the "keydown" event but not the "paste" event, so it's ok to process
            // it here and we don't need to do anything special in onPaste
            if ((event.metaKey || event.ctrlKey) && event.shiftKey && event.code === 'KeyV') {
                const pastedText = clipboard.readText();
                if (pastedText)
                    editor.insertContent(pastedText);
            }
        }
        editor.on('keyup', onKeyUp);
        editor.on('keydown', onKeyDown);
        editor.on('keypress', onKeypress);
        editor.on('paste', onPaste);
        // `compositionend` means that a user has finished entering a Chinese
        // (or other languages that require IME) character.
        editor.on('compositionend', onChangeHandler);
        editor.on('cut', onChangeHandler);
        editor.on('joplinChange', onChangeHandler);
        editor.on('Undo', onChangeHandler);
        editor.on('Redo', onChangeHandler);
        editor.on('ExecCommand', onExecCommand);
        return () => {
            try {
                editor.off('keyup', onKeyUp);
                editor.off('keydown', onKeyDown);
                editor.off('keypress', onKeypress);
                editor.off('paste', onPaste);
                editor.off('compositionend', onChangeHandler);
                editor.off('cut', onChangeHandler);
                editor.off('joplinChange', onChangeHandler);
                editor.off('Undo', onChangeHandler);
                editor.off('Redo', onChangeHandler);
                editor.off('ExecCommand', onExecCommand);
            }
            catch (error) {
                console.warn('Error removing events', error);
            }
        };
    }, [props.onWillChange, props.onChange, props.contentMarkupLanguage, props.contentOriginalCss, editor]);
    // -----------------------------------------------------------------------------------------
    // Destroy the editor when unmounting
    // Note that this effect must always be last, otherwise other effects that access the
    // editor in their clean up function will get an invalid reference.
    // -----------------------------------------------------------------------------------------
    react_1.useEffect(() => {
        return () => {
            if (editorRef.current)
                editorRef.current.remove();
        };
    }, []);
    // Currently we don't handle resource "auto" and "manual" mode with TinyMCE
    // as it is quite complex and probably rarely used.
    function renderDisabledOverlay() {
        const status = resourceHandling_1.resourcesStatus(props.resourceInfos);
        if (status === 'ready' && !draggingStarted)
            return null;
        const message = draggingStarted ? _('Drop notes or files here') : _('Please wait for all attachments to be downloaded and decrypted. You may also switch to %s to edit the note.', _('Code View'));
        const statusComp = draggingStarted ? null : React.createElement("p", { style: theme.textStyleMinor }, `Status: ${status}`);
        return (React.createElement("div", { style: styles.disabledOverlay },
            React.createElement("p", { style: theme.textStyle }, message),
            statusComp));
    }
    return (React.createElement("div", { style: styles.rootStyle },
        renderDisabledOverlay(),
        React.createElement("div", { style: { width: '100%', height: '100%' }, id: rootIdRef.current })));
};
exports.default = react_1.forwardRef(TinyMCE);
//# sourceMappingURL=TinyMCE.js.map