"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const react_1 = require("react");
// eslint-disable-next-line no-unused-vars
const TinyMCE_1 = require("./NoteBody/TinyMCE/TinyMCE");
const AceEditor_1 = require("./NoteBody/AceEditor/AceEditor");
const CodeMirror_1 = require("./NoteBody/CodeMirror/CodeMirror");
const react_redux_1 = require("react-redux");
const MultiNoteActions_1 = require("../MultiNoteActions");
const NoteToolbar_1 = require("../NoteToolbar/NoteToolbar");
const utils_1 = require("./utils");
const useSearchMarkers_1 = require("./utils/useSearchMarkers");
const useNoteSearchBar_1 = require("./utils/useNoteSearchBar");
const useMessageHandler_1 = require("./utils/useMessageHandler");
const useWindowCommandHandler_1 = require("./utils/useWindowCommandHandler");
const useDropHandler_1 = require("./utils/useDropHandler");
const useMarkupToHtml_1 = require("./utils/useMarkupToHtml");
const useFormNote_1 = require("./utils/useFormNote");
const styles_1 = require("./styles");
const types_1 = require("./utils/types");
const index_1 = require("../../lib/services/ResourceEditWatcher/index");
const CommandService_1 = require("../../lib/services/CommandService");
const { themeStyle } = require('lib/theme');
const NoteSearchBar = require('../NoteSearchBar.min.js');
const { reg } = require('lib/registry.js');
const { time } = require('lib/time-utils.js');
const markupLanguageUtils = require('lib/markupLanguageUtils');
const usePrevious = require('lib/hooks/usePrevious').default;
const Setting = require('lib/models/Setting');
const { _ } = require('lib/locale');
const Note = require('lib/models/Note.js');
const { bridge } = require('electron').remote.require('./bridge');
const ExternalEditWatcher = require('lib/services/ExternalEditWatcher');
const eventManager = require('lib/eventManager');
const NoteRevisionViewer = require('../NoteRevisionViewer.min');
const TagList = require('../TagList.min.js');
const commands = [
    require('./commands/showRevisions'),
];
function NoteEditor(props) {
    const [showRevisions, setShowRevisions] = react_1.useState(false);
    const [titleHasBeenManuallyChanged, setTitleHasBeenManuallyChanged] = react_1.useState(false);
    const [scrollWhenReady, setScrollWhenReady] = react_1.useState(null);
    const editorRef = react_1.useRef();
    const titleInputRef = react_1.useRef();
    const isMountedRef = react_1.useRef(true);
    const noteSearchBarRef = react_1.useRef(null);
    const formNote_beforeLoad = react_1.useCallback((event) => __awaiter(this, void 0, void 0, function* () {
        yield saveNoteIfWillChange(event.formNote);
        setShowRevisions(false);
    }), []);
    const formNote_afterLoad = react_1.useCallback(() => __awaiter(this, void 0, void 0, function* () {
        setTitleHasBeenManuallyChanged(false);
    }), []);
    const { formNote, setFormNote, isNewNote, resourceInfos } = useFormNote_1.default({
        syncStarted: props.syncStarted,
        noteId: props.noteId,
        isProvisional: props.isProvisional,
        titleInputRef: titleInputRef,
        editorRef: editorRef,
        onBeforeLoad: formNote_beforeLoad,
        onAfterLoad: formNote_afterLoad,
    });
    const formNoteRef = react_1.useRef();
    formNoteRef.current = Object.assign({}, formNote);
    const { localSearch, onChange: localSearch_change, onNext: localSearch_next, onPrevious: localSearch_previous, onClose: localSearch_close, setResultCount: setLocalSearchResultCount, showLocalSearch, setShowLocalSearch, searchMarkers: localSearchMarkerOptions, } = useNoteSearchBar_1.default();
    // If the note has been modified in another editor, wait for it to be saved
    // before loading it in this editor.
    // const waitingToSaveNote = props.noteId && formNote.id !== props.noteId && props.editorNoteStatuses[props.noteId] === 'saving';
    const styles = styles_1.default(props);
    function scheduleSaveNote(formNote) {
        if (!formNote.saveActionQueue)
            throw new Error('saveActionQueue is not set!!'); // Sanity check
        // reg.logger().debug('Scheduling...', formNote);
        const makeAction = (formNote) => {
            return function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const note = yield utils_1.formNoteToNote(formNote);
                    reg.logger().debug('Saving note...', note);
                    const savedNote = yield Note.save(note);
                    setFormNote((prev) => {
                        return Object.assign(Object.assign({}, prev), { user_updated_time: savedNote.user_updated_time });
                    });
                    ExternalEditWatcher.instance().updateNoteFile(savedNote);
                    props.dispatch({
                        type: 'EDITOR_NOTE_STATUS_REMOVE',
                        id: formNote.id,
                    });
                });
            };
        };
        formNote.saveActionQueue.push(makeAction(formNote));
    }
    function saveNoteIfWillChange(formNote) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!formNote.id || !formNote.bodyWillChangeId)
                return;
            const body = yield editorRef.current.content();
            scheduleSaveNote(Object.assign(Object.assign({}, formNote), { body: body, bodyWillChangeId: 0, bodyChangeId: 0 }));
        });
    }
    function saveNoteAndWait(formNote) {
        return __awaiter(this, void 0, void 0, function* () {
            saveNoteIfWillChange(formNote);
            return formNote.saveActionQueue.waitForAllDone();
        });
    }
    const markupToHtml = useMarkupToHtml_1.default({ themeId: props.theme, customCss: props.customCss });
    const allAssets = react_1.useCallback((markupLanguage) => __awaiter(this, void 0, void 0, function* () {
        const theme = themeStyle(props.theme);
        const markupToHtml = markupLanguageUtils.newMarkupToHtml({
            resourceBaseUrl: `file://${Setting.value('resourceDir')}/`,
        });
        return markupToHtml.allAssets(markupLanguage, theme);
    }), [props.theme]);
    const handleProvisionalFlag = react_1.useCallback(() => {
        if (props.isProvisional) {
            props.dispatch({
                type: 'NOTE_PROVISIONAL_FLAG_CLEAR',
                id: formNote.id,
            });
        }
    }, [props.isProvisional, formNote.id]);
    const previousNoteId = usePrevious(formNote.id);
    react_1.useEffect(() => {
        if (formNote.id === previousNoteId)
            return;
        if (editorRef.current) {
            editorRef.current.resetScroll();
        }
        setScrollWhenReady({
            type: props.selectedNoteHash ? types_1.ScrollOptionTypes.Hash : types_1.ScrollOptionTypes.Percent,
            value: props.selectedNoteHash ? props.selectedNoteHash : props.lastEditorScrollPercents[props.noteId] || 0,
        });
        index_1.default.instance().stopWatchingAll();
    }, [formNote.id, previousNoteId]);
    const onFieldChange = react_1.useCallback((field, value, changeId = 0) => {
        if (!isMountedRef.current) {
            // When the component is unmounted, various actions can happen which can
            // trigger onChange events, for example the textarea might be cleared.
            // We need to ignore these events, otherwise the note is going to be saved
            // with an invalid body.
            reg.logger().debug('Skipping change event because the component is unmounted');
            return;
        }
        handleProvisionalFlag();
        const change = field === 'body' ? {
            body: value,
        } : {
            title: value,
        };
        const newNote = Object.assign(Object.assign(Object.assign({}, formNote), change), { bodyWillChangeId: 0, bodyChangeId: 0, hasChanged: true });
        if (field === 'title') {
            setTitleHasBeenManuallyChanged(true);
        }
        if (isNewNote && !titleHasBeenManuallyChanged && field === 'body') {
            // TODO: Handle HTML/Markdown format
            newNote.title = Note.defaultTitle(value);
        }
        if (changeId !== null && field === 'body' && formNote.bodyWillChangeId !== changeId) {
            // Note was changed, but another note was loaded before save - skipping
            // The previously loaded note, that was modified, will be saved via saveNoteIfWillChange()
        }
        else {
            setFormNote(newNote);
            scheduleSaveNote(newNote);
        }
    }, [handleProvisionalFlag, formNote, isNewNote, titleHasBeenManuallyChanged]);
    useWindowCommandHandler_1.default({ dispatch: props.dispatch, formNote, setShowLocalSearch, noteSearchBarRef, editorRef, titleInputRef, saveNoteAndWait });
    const onDrop = useDropHandler_1.default({ editorRef });
    const onBodyChange = react_1.useCallback((event) => onFieldChange('body', event.content, event.changeId), [onFieldChange]);
    const onTitleChange = react_1.useCallback((event) => onFieldChange('title', event.target.value), [onFieldChange]);
    const onTitleKeydown = react_1.useCallback((event) => {
        const keyCode = event.keyCode;
        if (keyCode === 9) {
            // TAB
            event.preventDefault();
            if (event.shiftKey) {
                CommandService_1.default.instance().execute('focusElement', { target: 'noteList' });
            }
            else {
                CommandService_1.default.instance().execute('focusElement', { target: 'noteBody' });
            }
        }
    }, [props.dispatch]);
    const onBodyWillChange = react_1.useCallback((event) => {
        handleProvisionalFlag();
        setFormNote(prev => {
            return Object.assign(Object.assign({}, prev), { bodyWillChangeId: event.changeId, hasChanged: true });
        });
        props.dispatch({
            type: 'EDITOR_NOTE_STATUS_SET',
            id: formNote.id,
            status: 'saving',
        });
    }, [formNote, handleProvisionalFlag]);
    const onMessage = useMessageHandler_1.default(scrollWhenReady, setScrollWhenReady, editorRef, setLocalSearchResultCount, props.dispatch, formNote);
    const introductionPostLinkClick = react_1.useCallback(() => {
        bridge().openExternal('https://www.patreon.com/posts/34246624');
    }, []);
    const externalEditWatcher_noteChange = react_1.useCallback((event) => {
        if (event.id === formNote.id) {
            const newFormNote = Object.assign(Object.assign({}, formNote), { title: event.note.title, body: event.note.body });
            setFormNote(newFormNote);
        }
    }, [formNote]);
    const onNotePropertyChange = react_1.useCallback((event) => {
        setFormNote(formNote => {
            if (formNote.id !== event.note.id)
                return formNote;
            const newFormNote = Object.assign({}, formNote);
            for (const key in event.note) {
                if (key === 'id')
                    continue;
                newFormNote[key] = event.note[key];
            }
            return newFormNote;
        });
    }, []);
    react_1.useEffect(() => {
        eventManager.on('alarmChange', onNotePropertyChange);
        ExternalEditWatcher.instance().on('noteChange', externalEditWatcher_noteChange);
        return () => {
            eventManager.off('alarmChange', onNotePropertyChange);
            ExternalEditWatcher.instance().off('noteChange', externalEditWatcher_noteChange);
        };
    }, [externalEditWatcher_noteChange, onNotePropertyChange]);
    react_1.useEffect(() => {
        const dependencies = {
            setShowRevisions,
        };
        CommandService_1.default.instance().componentRegisterCommands(dependencies, commands);
        return () => {
            CommandService_1.default.instance().componentUnregisterCommands(commands);
        };
    }, [setShowRevisions]);
    const onScroll = react_1.useCallback((event) => {
        props.dispatch({
            type: 'EDITOR_SCROLL_PERCENT_SET',
            noteId: formNote.id,
            percent: event.percent,
        });
    }, [props.dispatch, formNote]);
    function renderNoNotes(rootStyle) {
        const emptyDivStyle = Object.assign({
            backgroundColor: 'black',
            opacity: 0.1,
        }, rootStyle);
        return React.createElement("div", { style: emptyDivStyle });
    }
    function renderNoteToolbar() {
        const toolbarStyle = {
            marginBottom: 0,
        };
        return React.createElement(NoteToolbar_1.default, { theme: props.theme, note: formNote, style: toolbarStyle });
    }
    function renderTagBar() {
        return props.selectedNoteTags.length ? React.createElement(TagList, { items: props.selectedNoteTags }) : null;
    }
    function renderTitleBar() {
        const titleBarDate = React.createElement("span", { style: styles.titleDate }, time.formatMsToLocal(formNote.user_updated_time));
        return (React.createElement("div", { style: { display: 'flex', flexDirection: 'row', alignItems: 'center' } },
            React.createElement("input", { type: "text", ref: titleInputRef, placeholder: props.isProvisional ? _('Creating new %s...', formNote.is_todo ? _('to-do') : _('note')) : '', style: styles.titleInput, onChange: onTitleChange, onKeyDown: onTitleKeydown, value: formNote.title }),
            titleBarDate));
    }
    const searchMarkers = useSearchMarkers_1.default(showLocalSearch, localSearchMarkerOptions, props.searches, props.selectedSearchId);
    const editorProps = {
        ref: editorRef,
        contentKey: formNote.id,
        style: styles.tinyMCE,
        onChange: onBodyChange,
        onWillChange: onBodyWillChange,
        onMessage: onMessage,
        content: formNote.body,
        contentMarkupLanguage: formNote.markup_language,
        contentOriginalCss: formNote.originalCss,
        resourceInfos: resourceInfos,
        htmlToMarkdown: utils_1.htmlToMarkdown,
        markupToHtml: markupToHtml,
        allAssets: allAssets,
        disabled: false,
        theme: props.theme,
        dispatch: props.dispatch,
        noteToolbar: null,
        onScroll: onScroll,
        setLocalSearchResultCount: setLocalSearchResultCount,
        searchMarkers: searchMarkers,
        visiblePanes: props.noteVisiblePanes || ['editor', 'viewer'],
        keyboardMode: Setting.value('editor.keyboardMode'),
        locale: Setting.value('locale'),
        onDrop: onDrop,
    };
    let editor = null;
    if (props.bodyEditor === 'TinyMCE') {
        editor = React.createElement(TinyMCE_1.default, Object.assign({}, editorProps));
    }
    else if (props.bodyEditor === 'AceEditor') {
        editor = React.createElement(AceEditor_1.default, Object.assign({}, editorProps));
    }
    else if (props.bodyEditor === 'CodeMirror') {
        editor = React.createElement(CodeMirror_1.default, Object.assign({}, editorProps));
    }
    else {
        throw new Error(`Invalid editor: ${props.bodyEditor}`);
    }
    const wysiwygBanner = props.bodyEditor !== 'TinyMCE' ? null : (React.createElement("div", { style: Object.assign({}, styles.warningBanner) },
        "This is an experimental WYSIWYG editor for evaluation only. Please do not use with important notes as you may lose some data! See the ",
        React.createElement("a", { style: styles.urlColor, onClick: introductionPostLinkClick, href: "#" }, "introduction post"),
        " for more information. TO SWITCH TO THE MARKDOWN EDITOR PLEASE PRESS \"Code View\"."));
    const noteRevisionViewer_onBack = react_1.useCallback(() => {
        setShowRevisions(false);
    }, []);
    if (showRevisions) {
        const theme = themeStyle(props.theme);
        const revStyle = Object.assign(Object.assign({}, props.style), { display: 'inline-flex', padding: theme.margin, verticalAlign: 'top', boxSizing: 'border-box' });
        return (React.createElement("div", { style: revStyle },
            React.createElement(NoteRevisionViewer, { customCss: props.customCss, noteId: formNote.id, onBack: noteRevisionViewer_onBack })));
    }
    if (props.selectedNoteIds.length > 1) {
        return React.createElement(MultiNoteActions_1.default, { theme: props.theme, selectedNoteIds: props.selectedNoteIds, notes: props.notes, dispatch: props.dispatch, watchedNoteFiles: props.watchedNoteFiles, style: props.style });
    }
    function renderSearchBar() {
        if (!showLocalSearch)
            return false;
        const theme = themeStyle(props.theme);
        return (React.createElement(NoteSearchBar, { ref: noteSearchBarRef, style: {
                display: 'flex',
                height: 35,
                borderTop: `1px solid ${theme.dividerColor}`,
            }, query: localSearch.query, searching: localSearch.searching, resultCount: localSearch.resultCount, selectedIndex: localSearch.selectedIndex, onChange: localSearch_change, onNext: localSearch_next, onPrevious: localSearch_previous, onClose: localSearch_close }));
    }
    function renderResourceWatchingNotification() {
        if (!Object.keys(props.watchedResources).length)
            return null;
        const resourceTitles = Object.keys(props.watchedResources).map(id => props.watchedResources[id].title);
        return (React.createElement("div", { style: styles.resourceWatchBanner },
            React.createElement("p", { style: styles.resourceWatchBannerLine },
                _('The following attachments are being watched for changes:'),
                " ",
                React.createElement("strong", null, resourceTitles.join(', '))),
            React.createElement("p", { style: Object.assign(Object.assign({}, styles.resourceWatchBannerLine), { marginBottom: 0 }) }, _('The attachments will no longer be watched when you switch to a different note.'))));
    }
    if (formNote.encryption_applied || !formNote.id || !props.noteId) {
        return renderNoNotes(styles.root);
    }
    return (React.createElement("div", { style: styles.root, onDrop: onDrop },
        React.createElement("div", { style: { display: 'flex', flexDirection: 'column', height: '100%' } },
            renderResourceWatchingNotification(),
            renderTitleBar(),
            React.createElement("div", { style: { display: 'flex', flexDirection: 'row', alignItems: 'center' } },
                renderNoteToolbar(),
                renderTagBar()),
            React.createElement("div", { style: { display: 'flex', flex: 1 } }, editor),
            React.createElement("div", { style: { display: 'flex', flexDirection: 'row', alignItems: 'center' } }, renderSearchBar()),
            wysiwygBanner)));
}
exports.NoteEditorComponent = NoteEditor;
const mapStateToProps = (state) => {
    const noteId = state.selectedNoteIds.length === 1 ? state.selectedNoteIds[0] : null;
    return {
        noteId: noteId,
        notes: state.notes,
        folders: state.folders,
        selectedNoteIds: state.selectedNoteIds,
        isProvisional: state.provisionalNoteIds.includes(noteId),
        editorNoteStatuses: state.editorNoteStatuses,
        syncStarted: state.syncStarted,
        theme: state.settings.theme,
        watchedNoteFiles: state.watchedNoteFiles,
        notesParentType: state.notesParentType,
        selectedNoteTags: state.selectedNoteTags,
        lastEditorScrollPercents: state.lastEditorScrollPercents,
        selectedNoteHash: state.selectedNoteHash,
        searches: state.searches,
        selectedSearchId: state.selectedSearchId,
        customCss: state.customCss,
        noteVisiblePanes: state.noteVisiblePanes,
        watchedResources: state.watchedResources,
    };
};
exports.default = react_redux_1.connect(mapStateToProps)(NoteEditor);
//# sourceMappingURL=NoteEditor.js.map