"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("../../../lib/services/ResourceEditWatcher/index");
const { bridge } = require('electron').remote.require('./bridge');
const Menu = bridge().Menu;
const MenuItem = bridge().MenuItem;
const Resource = require('lib/models/Resource.js');
const fs = require('fs-extra');
const { clipboard } = require('electron');
const { toSystemSlashes } = require('lib/path-utils');
const { _ } = require('lib/locale');
var ContextMenuItemType;
(function (ContextMenuItemType) {
    ContextMenuItemType["None"] = "";
    ContextMenuItemType["Image"] = "image";
    ContextMenuItemType["Resource"] = "resource";
    ContextMenuItemType["Text"] = "text";
    ContextMenuItemType["Link"] = "link";
})(ContextMenuItemType = exports.ContextMenuItemType || (exports.ContextMenuItemType = {}));
function resourceInfo(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const resource = options.resourceId ? yield Resource.load(options.resourceId) : null;
        const resourcePath = resource ? Resource.fullPath(resource) : '';
        return { resource, resourcePath };
    });
}
function menuItems() {
    return {
        open: {
            label: _('Open...'),
            onAction: (options) => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield index_1.default.instance().openAndWatch(options.resourceId);
                }
                catch (error) {
                    console.error(error);
                    bridge().showErrorMessageBox(error.message);
                }
            }),
            isActive: (itemType) => itemType === ContextMenuItemType.Image || itemType === ContextMenuItemType.Resource,
        },
        saveAs: {
            label: _('Save as...'),
            onAction: (options) => __awaiter(this, void 0, void 0, function* () {
                const { resourcePath, resource } = yield resourceInfo(options);
                const filePath = bridge().showSaveDialog({
                    defaultPath: resource.filename ? resource.filename : resource.title,
                });
                if (!filePath)
                    return;
                yield fs.copy(resourcePath, filePath);
            }),
            isActive: (itemType) => itemType === ContextMenuItemType.Image || itemType === ContextMenuItemType.Resource,
        },
        revealInFolder: {
            label: _('Reveal file in folder'),
            onAction: (options) => __awaiter(this, void 0, void 0, function* () {
                const { resourcePath } = yield resourceInfo(options);
                bridge().showItemInFolder(resourcePath);
            }),
            isActive: (itemType) => itemType === ContextMenuItemType.Image || itemType === ContextMenuItemType.Resource,
        },
        copyPathToClipboard: {
            label: _('Copy path to clipboard'),
            onAction: (options) => __awaiter(this, void 0, void 0, function* () {
                const { resourcePath } = yield resourceInfo(options);
                clipboard.writeText(toSystemSlashes(resourcePath));
            }),
            isActive: (itemType) => itemType === ContextMenuItemType.Image || itemType === ContextMenuItemType.Resource,
        },
        cut: {
            label: _('Cut'),
            onAction: (options) => __awaiter(this, void 0, void 0, function* () {
                clipboard.writeText(options.textToCopy);
                options.insertContent('');
            }),
            isActive: (_itemType, options) => !options.isReadOnly && (!!options.textToCopy || !!options.htmlToCopy),
        },
        copy: {
            label: _('Copy'),
            onAction: (options) => __awaiter(this, void 0, void 0, function* () {
                if (options.textToCopy) {
                    clipboard.writeText(options.textToCopy);
                }
                else if (options.htmlToCopy) {
                    clipboard.writeHTML(options.htmlToCopy);
                }
            }),
            isActive: (_itemType, options) => !!options.textToCopy || !!options.htmlToCopy,
        },
        paste: {
            label: _('Paste'),
            onAction: (options) => __awaiter(this, void 0, void 0, function* () {
                const content = clipboard.readHTML() ? clipboard.readHTML() : clipboard.readText();
                options.insertContent(content);
            }),
            isActive: (_itemType, options) => !options.isReadOnly && (!!clipboard.readText() || !!clipboard.readHTML()),
        },
        copyLinkUrl: {
            label: _('Copy Link Address'),
            onAction: (options) => __awaiter(this, void 0, void 0, function* () {
                clipboard.writeText(options.textToCopy);
            }),
            isActive: (itemType) => itemType === ContextMenuItemType.Link,
        },
    };
}
exports.menuItems = menuItems;
function contextMenu(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const menu = new Menu();
        const items = menuItems();
        if (!('readyOnly' in options))
            options.isReadOnly = true;
        for (const itemKey in items) {
            const item = items[itemKey];
            if (!item.isActive(options.itemType, options))
                continue;
            menu.append(new MenuItem({
                label: item.label,
                click: () => {
                    item.onAction(options);
                },
            }));
        }
        return menu;
    });
}
exports.default = contextMenu;
//# sourceMappingURL=contextMenu.js.map