"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Setting = require('lib/models/Setting');
const Note = require('lib/models/Note.js');
const BaseModel = require('lib/BaseModel.js');
const Resource = require('lib/models/Resource.js');
const { shim } = require('lib/shim');
const { bridge } = require('electron').remote.require('./bridge');
const ResourceFetcher = require('lib/services/ResourceFetcher.js');
const { reg } = require('lib/registry.js');
const joplinRendererUtils = require('lib/joplin-renderer').utils;
const { clipboard } = require('electron');
const mimeUtils = require('lib/mime-utils.js').mime;
const md5 = require('md5');
function handleResourceDownloadMode(noteBody) {
    return __awaiter(this, void 0, void 0, function* () {
        if (noteBody && Setting.value('sync.resourceDownloadMode') === 'auto') {
            const resourceIds = yield Note.linkedResourceIds(noteBody);
            yield ResourceFetcher.instance().markForDownload(resourceIds);
        }
    });
}
exports.handleResourceDownloadMode = handleResourceDownloadMode;
let resourceCache_ = {};
function clearResourceCache() {
    resourceCache_ = {};
}
exports.clearResourceCache = clearResourceCache;
function attachedResources(noteBody) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!noteBody)
            return {};
        const resourceIds = yield Note.linkedItemIdsByType(BaseModel.TYPE_RESOURCE, noteBody);
        const output = {};
        for (let i = 0; i < resourceIds.length; i++) {
            const id = resourceIds[i];
            if (resourceCache_[id]) {
                output[id] = resourceCache_[id];
            }
            else {
                const resource = yield Resource.load(id);
                const localState = yield Resource.localState(resource);
                const o = {
                    item: resource,
                    localState: localState,
                };
                // eslint-disable-next-line require-atomic-updates
                resourceCache_[id] = o;
                output[id] = o;
            }
        }
        return output;
    });
}
exports.attachedResources = attachedResources;
function commandAttachFileToBody(body, filePaths = null, options = null) {
    return __awaiter(this, void 0, void 0, function* () {
        options = Object.assign({ createFileURL: false, position: 0 }, options);
        if (!filePaths) {
            filePaths = bridge().showOpenDialog({
                properties: ['openFile', 'createDirectory', 'multiSelections'],
            });
            if (!filePaths || !filePaths.length)
                return null;
        }
        for (let i = 0; i < filePaths.length; i++) {
            const filePath = filePaths[i];
            try {
                reg.logger().info(`Attaching ${filePath}`);
                const newBody = yield shim.attachFileToNoteBody(body, filePath, options.position, {
                    createFileURL: options.createFileURL,
                    resizeLargeImages: 'ask',
                });
                if (!newBody) {
                    reg.logger().info('File attachment was cancelled');
                    return null;
                }
                body = newBody;
                reg.logger().info('File was attached.');
            }
            catch (error) {
                reg.logger().error(error);
                bridge().showErrorMessageBox(error.message);
            }
        }
        return body;
    });
}
exports.commandAttachFileToBody = commandAttachFileToBody;
function resourcesStatus(resourceInfos) {
    let lowestIndex = joplinRendererUtils.resourceStatusIndex('ready');
    for (const id in resourceInfos) {
        const s = joplinRendererUtils.resourceStatus(Resource, resourceInfos[id]);
        const idx = joplinRendererUtils.resourceStatusIndex(s);
        if (idx < lowestIndex)
            lowestIndex = idx;
    }
    return joplinRendererUtils.resourceStatusName(lowestIndex);
}
exports.resourcesStatus = resourcesStatus;
function handlePasteEvent(event) {
    return __awaiter(this, void 0, void 0, function* () {
        const output = [];
        const formats = clipboard.availableFormats();
        for (let i = 0; i < formats.length; i++) {
            const format = formats[i].toLowerCase();
            const formatType = format.split('/')[0];
            if (formatType === 'image') {
                if (event)
                    event.preventDefault();
                const image = clipboard.readImage();
                const fileExt = mimeUtils.toFileExtension(format);
                const filePath = `${Setting.value('tempDir')}/${md5(Date.now())}.${fileExt}`;
                yield shim.writeImageToFile(image, format, filePath);
                const md = yield commandAttachFileToBody('', [filePath]);
                yield shim.fsDriver().remove(filePath);
                if (md)
                    output.push(md);
            }
        }
        return output;
    });
}
exports.handlePasteEvent = handlePasteEvent;
//# sourceMappingURL=resourceHandling.js.map