"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const Note = require('lib/models/Note.js');
function useDropHandler(dependencies) {
    const { editorRef } = dependencies;
    return react_1.useCallback((event) => __awaiter(this, void 0, void 0, function* () {
        const dt = event.dataTransfer;
        const createFileURL = event.altKey;
        if (dt.types.indexOf('text/x-jop-note-ids') >= 0) {
            const noteIds = JSON.parse(dt.getData('text/x-jop-note-ids'));
            const noteMarkdownTags = [];
            for (let i = 0; i < noteIds.length; i++) {
                const note = yield Note.load(noteIds[i]);
                noteMarkdownTags.push(Note.markdownTag(note));
            }
            editorRef.current.execCommand({
                name: 'dropItems',
                value: {
                    type: 'notes',
                    markdownTags: noteMarkdownTags,
                },
            });
            return;
        }
        const files = dt.files;
        if (files && files.length) {
            const paths = [];
            for (let i = 0; i < files.length; i++) {
                const file = files[i];
                if (!file.path)
                    continue;
                paths.push(file.path);
            }
            editorRef.current.execCommand({
                name: 'dropItems',
                value: {
                    type: 'files',
                    paths: paths,
                    createFileURL: createFileURL,
                },
            });
        }
    }), []);
}
exports.default = useDropHandler;
//# sourceMappingURL=useDropHandler.js.map