"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const { themeStyle } = require('lib/theme');
const Note = require('lib/models/Note');
const Setting = require('lib/models/Setting');
const markupLanguageUtils = require('lib/markupLanguageUtils');
function useMarkupToHtml(dependencies) {
    const { themeId, customCss } = dependencies;
    return react_1.useCallback((markupLanguage, md, options = null) => __awaiter(this, void 0, void 0, function* () {
        options = Object.assign({ replaceResourceInternalToExternalLinks: false, resourceInfos: {} }, options);
        md = md || '';
        const theme = themeStyle(themeId);
        let resources = {};
        if (options.replaceResourceInternalToExternalLinks) {
            md = yield Note.replaceResourceInternalToExternalLinks(md, { useAbsolutePaths: true });
        }
        else {
            resources = options.resourceInfos;
        }
        delete options.replaceResourceInternalToExternalLinks;
        const markupToHtml = markupLanguageUtils.newMarkupToHtml({
            resourceBaseUrl: `file://${Setting.value('resourceDir')}/`,
        });
        const result = yield markupToHtml.render(markupLanguage, md, theme, Object.assign({}, {
            codeTheme: theme.codeThemeCss,
            userCss: customCss || '',
            resources: resources,
            postMessageSyntax: 'ipcProxySendToHost',
            splitted: true,
            externalAssetsOnly: true,
        }, options));
        return result;
    }), [themeId, customCss]);
}
exports.default = useMarkupToHtml;
//# sourceMappingURL=useMarkupToHtml.js.map