"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const contextMenu_1 = require("./contextMenu");
const index_1 = require("../../../lib/services/ResourceEditWatcher/index");
const BaseItem = require('lib/models/BaseItem');
const { _ } = require('lib/locale');
const BaseModel = require('lib/BaseModel.js');
const Resource = require('lib/models/Resource.js');
const { bridge } = require('electron').remote.require('./bridge');
const { urlDecode } = require('lib/string-utils');
const urlUtils = require('lib/urlUtils');
const ResourceFetcher = require('lib/services/ResourceFetcher.js');
const { reg } = require('lib/registry.js');
function useMessageHandler(scrollWhenReady, setScrollWhenReady, editorRef, setLocalSearchResultCount, dispatch, formNote) {
    return react_1.useCallback((event) => __awaiter(this, void 0, void 0, function* () {
        const msg = event.channel ? event.channel : '';
        const args = event.args;
        const arg0 = args && args.length >= 1 ? args[0] : null;
        // if (msg !== 'percentScroll') console.info(`Got ipc-message: ${msg}`, arg0);
        if (msg.indexOf('error:') === 0) {
            const s = msg.split(':');
            s.splice(0, 1);
            reg.logger().error(s.join(':'));
        }
        else if (msg === 'noteRenderComplete') {
            if (scrollWhenReady) {
                const options = Object.assign({}, scrollWhenReady);
                setScrollWhenReady(null);
                editorRef.current.scrollTo(options);
            }
        }
        else if (msg === 'setMarkerCount') {
            setLocalSearchResultCount(arg0);
        }
        else if (msg.indexOf('markForDownload:') === 0) {
            const s = msg.split(':');
            if (s.length < 2)
                throw new Error(`Invalid message: ${msg}`);
            ResourceFetcher.instance().markForDownload(s[1]);
        }
        else if (msg === 'contextMenu') {
            const menu = yield contextMenu_1.default({
                itemType: arg0 && arg0.type,
                resourceId: arg0.resourceId,
                textToCopy: arg0.textToCopy,
                htmlToCopy: '',
                insertContent: () => { console.warn('insertContent() not implemented'); },
            });
            menu.popup(bridge().window());
        }
        else if (msg.indexOf('joplin://') === 0) {
            const resourceUrlInfo = urlUtils.parseResourceUrl(msg);
            const itemId = resourceUrlInfo.itemId;
            const item = yield BaseItem.loadItemById(itemId);
            if (!item)
                throw new Error(`No item with ID ${itemId}`);
            if (item.type_ === BaseModel.TYPE_RESOURCE) {
                const localState = yield Resource.localState(item);
                if (localState.fetch_status !== Resource.FETCH_STATUS_DONE || !!item.encryption_blob_encrypted) {
                    if (localState.fetch_status === Resource.FETCH_STATUS_ERROR) {
                        bridge().showErrorMessageBox(`${_('There was an error downloading this attachment:')}\n\n${localState.fetch_error}`);
                    }
                    else {
                        bridge().showErrorMessageBox(_('This attachment is not downloaded or not decrypted yet'));
                    }
                    return;
                }
                try {
                    yield index_1.default.instance().openAndWatch(item.id);
                }
                catch (error) {
                    console.error(error);
                    bridge().showErrorMessageBox(error.message);
                }
            }
            else if (item.type_ === BaseModel.TYPE_NOTE) {
                dispatch({
                    type: 'FOLDER_AND_NOTE_SELECT',
                    folderId: item.parent_id,
                    noteId: item.id,
                    hash: resourceUrlInfo.hash,
                });
            }
            else {
                throw new Error(`Unsupported item type: ${item.type_}`);
            }
        }
        else if (urlUtils.urlProtocol(msg)) {
            if (msg.indexOf('file://') === 0) {
                // When using the file:// protocol, openExternal doesn't work (does nothing) with URL-encoded paths
                require('electron').shell.openExternal(urlDecode(msg));
            }
            else {
                require('electron').shell.openExternal(msg);
            }
        }
        else if (msg.indexOf('#') === 0) {
            // This is an internal anchor, which is handled by the WebView so skip this case
        }
        else {
            bridge().showErrorMessageBox(_('Unsupported link or message: %s', msg));
        }
    }), [dispatch, setLocalSearchResultCount, scrollWhenReady, formNote]);
}
exports.default = useMessageHandler;
//# sourceMappingURL=useMessageHandler.js.map