"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
function defaultLocalSearch() {
    return {
        query: '',
        selectedIndex: 0,
        resultCount: 0,
        searching: false,
        timestamp: 0,
    };
}
function useNoteSearchBar() {
    const [showLocalSearch, setShowLocalSearch] = react_1.useState(false);
    const [localSearch, setLocalSearch] = react_1.useState(defaultLocalSearch());
    const onChange = react_1.useCallback((query) => {
        setLocalSearch((prev) => {
            return {
                query: query,
                selectedIndex: 0,
                timestamp: Date.now(),
                resultCount: prev.resultCount,
                searching: true,
            };
        });
    }, []);
    const noteSearchBarNextPrevious = react_1.useCallback((inc) => {
        setLocalSearch((prev) => {
            const ls = Object.assign({}, prev);
            ls.selectedIndex += inc;
            ls.timestamp = Date.now();
            if (ls.selectedIndex < 0)
                ls.selectedIndex = ls.resultCount - 1;
            if (ls.selectedIndex >= ls.resultCount)
                ls.selectedIndex = 0;
            return ls;
        });
    }, []);
    const onNext = react_1.useCallback(() => {
        noteSearchBarNextPrevious(+1);
    }, [noteSearchBarNextPrevious]);
    const onPrevious = react_1.useCallback(() => {
        noteSearchBarNextPrevious(-1);
    }, [noteSearchBarNextPrevious]);
    const onClose = react_1.useCallback(() => {
        setShowLocalSearch(false);
        setLocalSearch(defaultLocalSearch());
    }, []);
    const setResultCount = react_1.useCallback((count) => {
        setLocalSearch((prev) => {
            if (prev.resultCount === count && !prev.searching)
                return prev;
            return Object.assign(Object.assign({}, prev), { resultCount: count, searching: false });
        });
    }, []);
    const searchMarkers = react_1.useCallback(() => {
        return {
            options: {
                selectedIndex: localSearch.selectedIndex,
                separateWordSearch: false,
                searchTimestamp: localSearch.timestamp,
            },
            keywords: [
                {
                    type: 'text',
                    value: localSearch.query,
                    accuracy: 'partially',
                },
            ],
        };
    }, [localSearch]);
    return { localSearch, onChange, onNext, onPrevious, onClose, setResultCount, showLocalSearch, setShowLocalSearch, searchMarkers };
}
exports.default = useNoteSearchBar;
//# sourceMappingURL=useNoteSearchBar.js.map