"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const react_1 = require("react");
const CommandService_1 = require("../../lib/services/CommandService");
const { connect } = require('react-redux');
const { buildStyle } = require('lib/theme');
const Toolbar = require('../Toolbar.min.js');
const Folder = require('lib/models/Folder');
const { _ } = require('lib/locale');
const { substrWithEllipsis } = require('lib/string-utils');
function styles_(props) {
    return buildStyle('NoteToolbar', props.theme, ( /* theme:any*/) => {
        return {
            root: Object.assign(Object.assign({}, props.style), { borderBottom: 'none' }),
        };
    });
}
function NoteToolbar(props) {
    const styles = styles_(props);
    const [toolbarItems, setToolbarItems] = react_1.useState([]);
    const selectedNoteFolder = Folder.byId(props.folders, props.note.parent_id);
    const folderId = selectedNoteFolder ? selectedNoteFolder.id : '';
    const folderTitle = selectedNoteFolder && selectedNoteFolder.title ? selectedNoteFolder.title : '';
    const cmdService = CommandService_1.default.instance();
    const updateToolbarItems = react_1.useCallback(() => {
        const output = [];
        output.push(cmdService.commandToToolbarButton('historyBackward'));
        output.push(cmdService.commandToToolbarButton('historyForward'));
        if (folderId && ['Search', 'Tag', 'SmartFilter'].includes(props.notesParentType)) {
            output.push({
                title: _('In: %s', substrWithEllipsis(folderTitle, 0, 16)),
                tooltip: folderTitle,
                iconName: 'fa-book',
                onClick: () => {
                    props.dispatch({
                        type: 'FOLDER_AND_NOTE_SELECT',
                        folderId: folderId,
                        noteId: props.note.id,
                    });
                },
            });
        }
        output.push(cmdService.commandToToolbarButton('showNoteProperties'));
        if (props.watchedNoteFiles.indexOf(props.note.id) >= 0) {
            output.push(cmdService.commandToToolbarButton('stopExternalEditing'));
        }
        else {
            output.push(cmdService.commandToToolbarButton('startExternalEditing'));
        }
        output.push(cmdService.commandToToolbarButton('editAlarm'));
        output.push(cmdService.commandToToolbarButton('setTags'));
        setToolbarItems(output);
    }, [props.note.id, folderId, folderTitle, props.watchedNoteFiles, props.notesParentType]);
    react_1.useEffect(() => {
        updateToolbarItems();
        cmdService.on('commandsEnabledStateChange', updateToolbarItems);
        return () => {
            cmdService.off('commandsEnabledStateChange', updateToolbarItems);
        };
    }, [updateToolbarItems]);
    return React.createElement(Toolbar, { style: styles.root, items: toolbarItems });
}
const mapStateToProps = (state) => {
    return {
        folders: state.folders,
        watchedNoteFiles: state.watchedNoteFiles,
        backwardHistoryNotes: state.backwardHistoryNotes,
        forwardHistoryNotes: state.forwardHistoryNotes,
        notesParentType: state.notesParentType,
    };
};
exports.default = connect(mapStateToProps)(NoteToolbar);
//# sourceMappingURL=NoteToolbar.js.map